from concurrent.futures import ThreadPoolExecutor
from fastapi import APIRouter, HTTPException
from fastapi.responses import JSONResponse
import azure.cosmos.exceptions as exceptions
from ..database.mycosmosdb import callConnect
import concurrent.futures
from typing import List, Optional
from ..storage.upload import delete_file_from_azure_blob
from app.models.schemas import Time, CBIVImpactSummary, CBIVFileDelete
from .utils import read_item, get_metrics, get_all_data_for_year_qtr
import logging
from pydantic import BaseModel

router = APIRouter()


@router.post("/impact-management")
async def save_cbiv_impact_management_company_summary_data(measurement: CBIVImpactSummary):
    """
    Saves the CBIV Impact Management Company Summary data to the database.

    Args:
    - measurement: a CBIVImpactSummary object

    Returns:
    - a JSONResponse with the saved measurement data

    Raises:
    - HTTPException with status_code 404 if the specified data is not found in the database
    - HTTPException with status_code 400 if the specified category is not accepted
    """

    db_result = False
    try:
        db_result = read_item(callConnect.container15,
                              measurement.oid, measurement.email)
    except exceptions.CosmosHttpResponseError as e:
        raise HTTPException(status_code=404, detail="Data not found")

    metric = measurement.metric

    # extracting total and varialbes
    ignore_keys = ["oid", "email", "year", "quarter", "metric"]
    data = {k: v for k, v in measurement.dict(
    ).items() if v is not None and k not in ignore_keys}

    summary_data = {"label": "1", "value": data}
    quarter_data = {"label": measurement.quarter, "metrics": [summary_data]}
    year_data = {"label": measurement.year, "value": [quarter_data]}
    metric_data = {"label": metric, "metrics": [year_data]}

    # check if custom measure exists
    if not db_result or "custommeasure" not in db_result:
        return None

    # user dict
    user = {"id": db_result["id"],
            "email": db_result["email"], "custommeasure": []}
    custom_measure = db_result["custommeasure"]

    # check if renewal-measurement data exists in custom measure
    result = next(
        (x for x in custom_measure if x["label"] == metric), None)
    if result:

        other_result = [x for x in custom_measure if x["label"] != metric]

        # find data with given year
        result_year = next(
            (x for x in result["metrics"] if x["label"] == measurement.year), None)

        if result_year:
            other_result_years = [
                x for x in result["metrics"] if x["label"] != measurement.year]

            # if data with given year exists -> append quarter data with given quarter and metrics
            other_quarters = [x for x in result_year["value"]
                              if x["label"] != measurement.quarter]

            other_quarters.append(quarter_data)

            result_year["value"] = other_quarters
            other_result_years.append(result_year)
        else:
            # if data with given year doesn't exist -> append year data with given quarter and metrics
            result["metrics"].append(year_data)

        user["custommeasure"] = other_result + [result]

    else:
        # append complete metric data if metric doesnot exist
        custom_measure.append(metric_data)
        user["custommeasure"] = custom_measure

    callConnect.container15.upsert_item(body=user)

    return JSONResponse(content=summary_data)


@router.delete("/delete-file")
async def delete_cbiv_file(measurement: CBIVFileDelete):
    """
    Deletes the CBIV file with the given file key and metric from the database.
    """
    db_result = False
    try:
        db_result = read_item(callConnect.container15, measurement.oid, measurement.email)
    except exceptions.CosmosHttpResponseError as e:
        raise HTTPException(status_code=404, detail="Data not found")

    metric = measurement.metric
    key_lookup = {
        "fileKPIs": "kpi",
        "fileHumanRights": "humanRights",
        "fileAntiCorruption": "antiCorruption",
        "fileEthics": "ethics",
        "fileDEI": "dei",
        "fileESG": "esg",
        "fileOthers": "others",
        "filePublications": "publications",
        "fileDocs": "docs",
        "fileWhistleblower": "whistleblower",
        "fileSupplier": "supplier",
        "fileAccidentPrevention": "accidentPrevention",
        "fileHumanTrafficking": "humanTrafficking",
        "fileDueDiligence": "dueDiligence",
        "fileMonitorCompliance": "monitorCompliance",
        "fileData": "data",
        "fileFamily": "family",
        "fileEmployee": "employee"
    }
    if measurement.key not in key_lookup:
        raise HTTPException(status_code=400, detail="Invalid file key")
    file_key = key_lookup[measurement.key]

    try:
        # check if custom measure exists
        if not db_result or "custommeasure" not in db_result:
            return None

        # user dict
        user = {"id": db_result["id"],
                "email": db_result["email"], "custommeasure": []}
        custom_measure = db_result["custommeasure"]

        # check if renewal-measurement data exists in custom measure
        result = next(
            (x for x in custom_measure if x["label"] == metric), None)
        if result:

            other_result = [x for x in custom_measure if x["label"] != metric]

            # find data with given year
            result_year = next(
                (x for x in result["metrics"] if x["label"] == measurement.year), None)

            if result_year:
                other_result_years = [
                    x for x in result["metrics"] if x["label"] != measurement.year]

                # if data with given year exists
                quarter = next(
                    (x for x in result_year["value"] if x["label"] == measurement.quarter), None)

                other_quarters = [x for x in result_year["value"]
                                  if x["label"] != measurement.quarter]

                if quarter:
                    # if quarter exists -> update metrics
                    previous_key_value = quarter["metrics"][0]["value"][0][file_key]
                    if previous_key_value["file_name"] == " ":
                        return JSONResponse(content=f"File for {file_key} under {metric} doesn't exist", status_code=400)

                    # deleting file with that key
                    delete_file_from_azure_blob(previous_key_value["file_name"])

                    # updating value
                    quarter["metrics"][0]["value"][0][file_key] = {
                        "file_name": " ", "file_url": " "}
                        
                    other_quarters.append(quarter)

                    result_year["value"] = other_quarters

                    other_result_years.append(result_year)

                    user["custommeasure"] = other_result + [result]

                    callConnect.container15.upsert_item(body=user)

                    return JSONResponse(content=f"File for {file_key} under {metric} deleted successfully")

    except Exception as e:
        logging.error(e)
        return JSONResponse(content="Internal server error", status_code=400)


class CBIVImpactMeasure(BaseModel):
    oid: str
    email: str
    metric: str
    current_year: str
    current_qtr: str
    prev_year: str
    prev_qtr: str


@router.post('/custommeasure-copy-data')
async def cbiv_impact_copy_data(cmet: CBIVImpactMeasure):
    """
    Endpoint to copy data from the most recent available year/quarter to the 
    current selected year/quarter for the impact section of CBIV, 
    only if there is no data already present for the current year/quarter.
    """
    try:
        db_result = read_item(callConnect.container15, cmet.oid, cmet.email)
    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /custommeasure')
        raise HTTPException(status_code=500, detail="Internal Server Error")


    if "custommeasure" in db_result:
        custom_measure = db_result["custommeasure"]

        user = {
            "id": db_result["id"],
            "email": db_result["email"],
            "custommeasure": []
        }

        # check if given metric exists, for eg: it might be for CorporateImpact
        result = next((x for x in custom_measure if x["label"] == cmet.metric), None)
        
        if result is None:
            return JSONResponse(content="Metric not found", status_code=400)

        # print(result)
        # print(cmet)
        # check if the data already exists for the current year and quarter
        current_yr_qtr_data = get_all_data_for_year_qtr(result, cmet.current_year, cmet.current_qtr)
        # get data for previous year and quarter
        prev_yr_qtr_data = get_all_data_for_year_qtr(result, cmet.prev_year, cmet.prev_qtr)

        if current_yr_qtr_data is None and prev_yr_qtr_data is not None:

            metrics_data = {"label": "1", "value": prev_yr_qtr_data}
            quarter_data = {"label": cmet.current_qtr, "metrics":[metrics_data] }
            year_data = {"label": cmet.current_year, "value": [quarter_data]}

            # retrieve other metrics data
            other_metrics = [x for x in custom_measure if x["label"] != cmet.metric]

            # find data for current year
            result_year = next(
                (x for x in result["metrics"] if x["label"] == cmet.current_year), None)

            if result_year:
                other_years_results = [
                    x for x in result["metrics"] if x["label"] != cmet.current_year]

                # if data with given year exists -> append quarter data with 
                # given quarter and metrics
                other_quarters = [x for x in result_year["value"]
                                if x["label"] != cmet.current_qtr]

                other_quarters.append(quarter_data)

                result_year["value"] = other_quarters
                other_years_results.append(result_year)
            else:
                # if data with given year doesn't exist 
                # -> append year data with given quarter and metrics
                result["metrics"].append(year_data)

            user["custommeasure"] = other_metrics + [result]

            callConnect.container15.upsert_item(body=user)

            return JSONResponse(content="Data copied successfully", status_code=200)
        else:
            return JSONResponse(content="Data already exists", status_code=200)
    else:

        return JSONResponse(content="Data not found", status_code=400)







